/* Ball Ballencer (two set points) with PID Fiddler
 *  By: Dave Gundlach
 *  2/18/18
 *  
 *  The Ball Ballencer is a PID test rig to learn how to tune PID settings.  The PID Fiddler
 *  is used to change the PID settings without reflashing this skectch when settings are changed.
 *  The PID Fiddler sends new settings by serial from the PID Fiddler Arduino to the Ball Ballencer
 *  Arduino.  The PID Fiddler does not need to be connected for this skecth to function.
 *  
 *  Ball Ballencer is an Aluminium Angle about 500mm long, set on a 45 degree angle to form
 *  a channel for a pingpong ball.  The angle is on a pivot so it can rock back and forth.
 *  At one end is an Adafruit VL35L0X breakout board.  The VL53L0X is a distance sensor to 
 *  measure the distance of the ball down the rail.  The rail is tipped back and forth with
 *  a servo.  The inside of the angle is painted flat black to reduce reflections of the laser 
 *  light that reduces semsor error.
 *  
 *  This sketch changes the set point between two points after a set time.  The serial plotter
 *  can be used to plot the movement of the ball.  Watch for overshoot and bias when set point changes.
 *  Adjust tunings to get the desired response.
 *  
 *  Pinout:
 *  1 - Serial Tx from PID Fiddler (remove when uploading sketch)
 *  9 - Servo Data
 *  11 - Green LED - Sitpoint 2 indicator
 *  12 - Red LED  - Setpoint 1 indicator
 *  13 - Onboard LED - Serial input indicator
 *  Note: Servo powered from seperate power supply.  Connect servo power supply ground and PID Fiddler ground
 *  
 *  Tips - This test rig uses distance of the ball from the end of the rail.  These values range from 0-500mm (aprox).
 *  This distance value is the "Input" to the PID.  The "Output is to a servo (0-180).   Values for Kp, Ki, and Kd seem 
 *  to range in the 0-1 range.  Depending on the range of input and values and the range of the output values
 *  affect the range of values of Kp, Ki, and Kd.  If the number range is smaller, the values may need to be larger.
 *  if the output values have a larger range, the vaues may need to be smaller.
 *  
 *  SetSampleTime: The sensor values jitter a bit with a still target.  If the sample time is too small, the Output jitters
 *  from this noise from the sensor.  Using larger sample time will average the values and smooth the output.  But making
 *  the sample times too large will make the corrections jerky.  Experiment with SetSampleTime values to get the desired
 *  output.
 */

#include "Adafruit_VL53L0X.h"  // Distance Measurement Library 
#include <Servo.h>             // Servo Library
#include <PID_v1.h>            // PID Library

Adafruit_VL53L0X lox = Adafruit_VL53L0X();   

Servo myservo;                      // create servo object to control a servo
double Setpoint, Input, Output;     // PID Variables
double Kp = .3, Ki = .04, Kd = .2;  //.3,.15,.4
int sTime = 200;

int LED = 13;           // PID Fiddler data transfer LED
int RLED = 12;          // Setpoint 1 active LED
int GLED = 11;          // Setpoint 2 active LED
int serialCount = 0;    // Serial Data Counter
int sp1 = 200;          // Set Point 1
int sp2 = 300;          // Set Point 2

PID myPID(&Input, &Output, &Setpoint, Kp, Ki, Kd, DIRECT);
unsigned long timer = 0;      // Not Used
unsigned long prevTime = 0;   // Set Point Timer
const long interval = 15000;  // Set Point Timer Interval
int setPos = LOW;             // Set Point Position 

void setup() {
  Serial.begin(115200);
  pinMode(LED, OUTPUT);
  pinMode(RLED, OUTPUT);
  pinMode(GLED, OUTPUT);
  while (! Serial) {          // wait until serial port opens for native USB devices
    delay(1);
  }
  if (!lox.begin()) {         // Setup for L0X distance measurement
    Serial.println(F("Failed to boot VL53L0X"));
    while (1);
  }
  myservo.attach(9);                // attaches the servo on pin 9 to the servo object
  myPID.SetMode(AUTOMATIC);
  myPID.SetOutputLimits(5, 175);    // Servo Arm min/max positions
  myPID.SetSampleTime(sTime);       // 200 ** Smaller Values produce more jitter from the sensor, larger values smooth jitter
  Setpoint = sp1;                   // initial setpoint 1
  digitalWrite(RLED, HIGH);         // Turn on red LED setpoint indicator
  digitalWrite(GLED, LOW);          // Turn off green LED setpoint indicator
}

void loop() {
  unsigned long currentTime = millis();         // Timer for switching setpoints
  if (currentTime - prevTime >= interval) {     // Change setpoint
    prevTime = currentTime;
    if (setPos == LOW) {
      setPos = HIGH;
      Setpoint = sp2;                           // New setpoint value 2
      digitalWrite(RLED, LOW);                  // Turn Red LED Off
      digitalWrite(GLED, HIGH);                 // Turn Green LED On
    }
    else {                            
      setPos = LOW;
      Setpoint = sp1;                            // New setpoint value 1
      digitalWrite(RLED, HIGH);                  // Turn Red LED On
      digitalWrite(GLED, LOW);                   // Turn Green LED Off
    }

  }
  readSerial();                                   // Check for data from PID Fiddler
  Input = int(ReadDist());                        // Read distance (location of ball)
  myPID.Compute();                                // PID Compute
  myservo.write(Output);                          // sets the servo position according to the scaled value

}
int ReadDist() {                                  // Reads distance to ball, returns distance as integer in mm
  VL53L0X_RangingMeasurementData_t measure;

  //  Serial.print("Reading a measurement... ");  // Not used for plotter
  lox.rangingTest(&measure, false);               // pass in 'true' to get debug data printout!

  if (measure.RangeStatus != 4) {                 // phase failures have incorrect data
    //    Serial.print("Distance (mm): ");        // Not used for plotter
    Serial.println(measure.RangeMilliMeter);
  } else {
    //    Serial.println(" out of range ");       // Not used for plotter
  }

  int range = measure.RangeMilliMeter;            // Read Distance
  return range;                                   // Return Distance

}
void readSerial() {                       // Read serial from PID Fiddler
  int val;                                // serial value
  int incoming = Serial.available();      // check if data is waiting

  if (incoming > 0) {                     // if data is available
    serialCount = serialCount + 1;        // count packets of data (1-Kp,2-0,3-Ki,4-0,5-Kd,6-0,7-0)

    //    timer=millis()+500;             // Not used, could be used for timeout of serial data
    val = Serial.parseInt();              //Reads integers as integer rather than ASCI. Anything else returns 0
//        Serial.print("Raw Value= "); Serial.println(val);           // Not used for Serial Plotter
//        Serial.print("Serial Count= ");Serial.println(serialCount); // Not used for Serial Plotter
    if (val < 500 && serialCount == 1) {                 // if value is less than 1000 (or 500) this is Kd value
      val = constrain(val, 0, 100);                      // assure value is constrained to proper value
      Kp = double(val) / 100;
    }
    else if (val < 1500 && serialCount == 3 ) {           // if value is less than 2000, it's Ki value
      val = val - 1000;                                   // subtract 1000 (PID Fiddler adds 1000 to second value
      val = constrain(val, 0, 100);                       // assure value is proper value
      Ki = double(val) / 100;
    }
    else if (val < 2500 && serialCount == 5) {            // if value is less than 3000, it's Kd value
      val = val - 2000;                                   // subtract 2000 (PID Fiddler adds 2000 to third value
      val = constrain(val, 0, 100);                       // assure value is proper
      Kd = double(val) / 100;
    }
    else if (val < 3500 && serialCount == 7) {
      val = val - 3000;
      val = constrain(val, 10, 600);
      sTime = val;
    }
    if (serialCount > 7) {                                 // All PID values recieved
      myPID.SetTunings(Kp, Ki, Kd);                        // Set new tuneing values
      myPID.SetSampleTime(sTime);                          // Set new sample time value
//      Serial.print("Kp= ");Serial.println(Kp);   // not used for plotter
//      Serial.print("Ki= ");Serial.println(Ki);
//      Serial.print("Kd= ");Serial.println(Kd);
//      Serial.print("ST= ");Serial.println(sTime);
//      delay(1000);
      serialCount = 0;                                     // reset serial data counter
      digitalWrite(LED, LOW);                              // Turn Off data LED
    }
  }
  if (serialCount > 0) digitalWrite(LED, HIGH);             // Turn On data LED

}

